unit PrsList;

interface

uses Classes,Windows,SysUtils;

type
 TProcess=class
  public
   ThreadCount         : dword;
   CreationTime        : TFileTime;
   ProcessName         : string;
   BasePriority        : dword;
   ProcessID           : dword;
   ParentProcessID     : dword;
   HandleCount         : dword;
   VirtualBytesPeak    : dword;
   VirtualBytes        : dword;
   PageFaults          : dword;
   WorkingSetPeak      : dword;
   WorkingSet          : dword;
   PagedPool           : dword;
   NonPagedPool        : dword;
   PageFileBytesPeak   : dword;
   PageFileBytes       : dword;
   PrivateBytes        : dword;
 end;

 TProcessList=class(TList)
  protected
   function GetProcess(idx : integer) : TProcess;
  public
   constructor Create;
   destructor Destroy;override;
   procedure Refresh;
   property Processes[idx : integer] : TProcess read GetProcess;
 end;

implementation

function NtQuerySystemInformation(dt : dword;
  buf : pointer;
  bufsize : dword;
  retlen : pointer) : dword;stdcall;external 'ntdll.dll';

type
 PBA = ^TBA;
 TBA = array[0..1000000] of byte;

 PThreadInfo = ^TThreadInfo;
 TThreadInfo = record
  ftCreationTime : TFileTime;
  dwUnknown1 : dword;
  dwStartAddress : dword;
  dwOwningPID : dword;
  dwThreadID : dword;
  dwCurrentPriority : dword;
  dwBasePriority : dword;
  dwContextSwitches : dword;
  dwThreadState : dword;
  dwUnknown2 : dword;
  dwUnknown3 : dword;
  dwUnknown4 : dword;
  dwUnknown5 : dword;
  dwUnknown6 : dword;
  dwUnknown : dword;
 end;

 PProcessInfo = ^TProcessInfo;
 TProcessInfo=record
  dwOffset            : dword; // an ofset to the next Process structure
  dwThreadCount       : dword;
  dwUnkown1           : array[0..5] of dword;
  ftCreationTime      : TFileTime;
  dwUnkown2           : dword;
  dwUnkown3           : dword;
  dwUnkown4           : dword;
  dwUnkown5           : dword;
  dwUnkown6           : dword;
  pszProcessName      : PWideChar;
  dwBasePriority      : dword;
  dwProcessID         : dword;
  dwParentProcessID   : dword;
  dwHandleCount       : dword;
  dwUnkown7           : dword;
  dwUnkown8           : dword;
  dwVirtualBytesPeak  : dword;
  dwVirtualBytes      : dword;
  dwPageFaults        : dword;
  dwWorkingSetPeak    : dword;
  dwWorkingSet        : dword;
  dwUnkown9           : dword;
  dwPagedPool         : dword; // kbytes
  dwUnkown10          : dword;
  dwNonPagedPool      : dword; // kbytes
  dwPageFileBytesPeak : dword;
  dwPageFileBytes     : dword;
  dwPrivateBytes      : dword;
  dwUnkown11          : dword;
  dwUnkown12          : dword;
  dwUnkown13          : dword;
  dwUnkown14          : dword;
  ThreadInfo : PThreadInfo; // Thread list
 end;

function TProcessList.GetProcess;
begin
 Result:=TProcess(Items[idx]);
end;

constructor TProcessList.Create;
begin
 inherited Create;
end;

destructor TProcessList.Destroy;
var
 i : integer;
begin
 if Count>0 then
  for i:=0 to Count-1 do
   TProcess(Items[i]).Free;
 inherited Destroy;
end;

procedure TProcessList.Refresh;
var
 i,rl,cp : dword;
 pinfo : PProcessInfo;
 buf : PBA;
 pcs : TProcess;
begin
 if Count>0 then
  for i:=0 to Count-1 do
   TProcess(Items[i]).Free;
 Clear;
 GetMem(buf,$10000);
 rl:=0;
 i:=NtQuerySystemInformation(5,buf,$10000,@rl);
 if i=0 then begin
  cp:=0;
  repeat
   pinfo:=PProcessInfo(@buf[cp]);
   cp:=cp+pinfo.dwOffset;
   pcs:=TProcess.Create;
   with pinfo^ do begin
    pcs.ThreadCount:=dwThreadCount;
    pcs.CreationTime:=ftCreationTime;
    if pszProcessName<>nil then
     pcs.ProcessName:=WideCharToString(pszProcessName)
    else
     pcs.ProcessName:='System Idle';
    pcs.BasePriority:=dwBasePriority;
    pcs.ProcessID:=dwProcessID;
    pcs.ParentProcessID:=dwParentProcessID;
    pcs.HandleCount:=dwHandleCount;
    pcs.VirtualBytesPeak:=dwVirtualBytesPeak;
    pcs.VirtualBytes:=dwVirtualBytes;
    pcs.PageFaults:=dwPageFaults;
    pcs.WorkingSetPeak:=dwWorkingSetPeak;
    pcs.WorkingSet:=dwWorkingSet;
    pcs.PagedPool:=dwPagedPool;
    pcs.NonPagedPool:=dwNonPagedPool;
    pcs.PageFileBytesPeak:=dwPageFileBytesPeak;
    pcs.PageFileBytes:=dwPageFileBytes;
    pcs.PrivateBytes:=dwPrivateBytes;
    Add(pcs);
   end;
  until pinfo.dwOffset=0;
 end;
 FreeMem(buf);
end;

end.
